package gaphicsExamples;
import javax.swing.*;
import java.awt.* ;
import java.awt.geom.*;

class MiscGraphicsInternalFrame extends JInternalFrame {
	
}

class MiscGraphicsDeskTop extends JDesktopPane {
	
	ImageIcon icon ;
	
	MiscGraphicsDeskTop() {
		// The "logo.gif" file should reside in the same directory
		// as the .class file for this class.
		// If the .class file is buried in a .jar file, then the
		// .gif file should be in the same directory within the .jar
		// file.
		java.net.URL imgURL = this.getClass().getResource("logo.gif");
		icon = new ImageIcon( imgURL ) ; 
		
		// Make an internal frame for this desktop
		MiscGraphicsInternalFrame frame = new MiscGraphicsInternalFrame();
		frame.setSize(100, 100) ;
        frame.setVisible(true);
        frame.setResizable(true) ;
        frame.setMaximizable(true) ;
        add(frame);
        
		setDragMode(JDesktopPane.OUTLINE_DRAG_MODE);
	}
	
	@Override public void paintComponent( Graphics g ) {
		super.paintComponent( g ) ;
		Color color_orig = g.getColor() ;
		Font font_orig = g.getFont() ;
		
		// Rectangles
		
		// The wrong way to put a blue border around an
		// orange rectangle
		g.setColor( Color.BLUE) ;
		int xr = 10, yr = 20, wr = 15, hr=15 ;
		g.drawRect( xr, yr, wr, hr ) ;
		g.setColor( Color.ORANGE) ;
		g.fillRect( xr, yr, wr, hr ) ;
		xr += 40 ;
		
		// The right way (according to Java's documentation)
		g.setColor( Color.BLUE) ;
		g.drawRect( xr, yr, wr+1, hr+1 ) ;
		g.setColor( Color.ORANGE) ;
		g.fillRect( xr+1, yr+1, wr, hr ) ;
		xr += 40 ;
		
		// What actually works on my Win7 implementation under
		// VMWare and OS X.
		g.setColor( Color.BLUE) ;
		g.drawRect( xr, yr, wr+1, hr+1 ) ;
		g.setColor( Color.ORANGE) ;
		g.fillRect( xr+1, yr, wr, hr ) ;
		
		// Fonts and font metrics
		// Put Happy Birthday in a box.
		// Here I sized the box according to the size of
		// the strings in size 20 font.
		// See my notes on how to scale the font so that it
		// will fit in a box of given size.
		String[] lines = new String[]{ "Happy", "Birthday", "to You" } ;
		Font font = new Font(Font.SERIF, Font.BOLD, 20) ;
		g.setFont( font ) ;
		FontMetrics fm = g.getFontMetrics(font) ;
		int w = 0 ;
		for( String line : lines) 
			w = Math.max(w, fm.stringWidth(line) ) ;
		int k = lines.length ;
		int h = k * fm.getHeight() - fm.getLeading() ;
		int x = 100 ;
		int y = 100 ;
		System.out.println( "Ascent  ="+ fm.getAscent()) ;
		System.out.println( "Descent ="+ fm.getDescent()) ;
		System.out.println( "Leading ="+ fm.getLeading()) ;
		System.out.println( "Height  ="+ fm.getHeight()) ;
		g.setColor( Color.YELLOW) ;
		g.fillRect(x, y, w, h) ;
		int d = fm.getDescent() ;
		for( String line : lines) {
			g.setColor( Color.BLACK) ;
			int y1 = y + fm.getAscent() ;
			g.drawString(line, x, y1) ;
			g.setColor( Color.BLUE ) ;
			// Show the location of 
			// (a) the top line
			g.drawLine(x, y, x+w, y) ;
			// (b) the baseline
			g.drawLine(x, y1, x+w, y1) ;
			// (c) the bottom line
			y1 += fm.getDescent() ;
			g.drawLine(x, y1, x+w, y1) ;
			y += fm.getHeight() ;
		}
		// NB on my machine the leading for this
		//    font is only 1, so it is hard to see that
		//    there are 6 distinct horizontal lines.
		
		// Now a bit of fun with font transformation
		// (a) Draw the message in black
		String message = "Happy Birthday" ;
		x = 100 ; y = 200 ;
		g.setColor( Color.BLACK) ;
		g.drawString(message, x, y) ;
		
		// Now draw its shadow.
		AffineTransform xform = new AffineTransform() ;
		xform.scale(1.0, -1.0) ; // Flip on x axis
		xform.shear(-1.0, 0.0) ; // Shear
		g.setFont( font.deriveFont(xform) ) ;
		
		g.setColor( Color.DARK_GRAY) ;
		g.drawString(message, x+d, y+2*d) ;

		
		// Images
		x = 50 ; y = 200 ;
		g.drawImage(icon.getImage(), x, y, null) ;
		// 
		g.setColor( Color.CYAN) ;
		g.drawRect(x, y, icon.getIconWidth(), icon.getIconHeight()) ;
		
		g.setColor( Color.BLUE) ;
		g.drawLine(x-10, y, x+10, y) ;
		g.drawLine(x, y-10, x, y+10) ;
		
		
		g.setColor( color_orig ) ;
		g.setFont( font_orig ) ;

		// Shapes
		x = 170; y = 250 ;
		w = 30; h = 20 ;
		Point2D p0 = new Point2D.Double(x, y);
		Point2D p1 = new Point2D.Double(x+w, y+h) ;
		Shape l = new Line2D.Double( p0, p1 ) ;
		Shape r = new Rectangle2D.Double( x, y, w, h ) ;
		Shape e = new Ellipse2D.Double( x, y, w, h ) ;
		Path2D p = new Path2D.Double() ;
		p.append( r, false ) ;
		p.append( e, false ) ;
		
		
		Graphics2D g2d = (Graphics2D) g ;
		
		g.setColor( Color.WHITE) ;
		g2d.fill( r ) ;
		g.setColor( Color.RED) ;
		g2d.fill( e ) ;
		g.setColor( Color.BLACK) ;
		g2d.draw( l ) ;
		g2d.draw( r ) ;
		g2d.draw( e ) ;
		g2d.draw( p ) ;
		

		// Put the state of g back the way it was.
		g.setColor( color_orig ) ;
		g.setFont( font_orig ) ;
	}
}

public class MiscGraphics extends JFrame {
	
	JDesktopPane panel = new MiscGraphicsDeskTop() ;
	
	public MiscGraphics() {
		setDefaultCloseOperation(EXIT_ON_CLOSE) ;
		add( panel ) ;
		setSize( 300, 350 ) ;
		setVisible( true ) ;
	}
	
	public static void main(String[] args) {
		SwingUtilities.invokeLater( new Runnable() {
			@Override public void run() {
				new MiscGraphics() ;
			}} ) ;
	}
}
